--
-- @package		ADOS - Automatic Downloading System
-- @version		1.3.0
--
-- @author		DINI
-- @copyright	2007—2008
--
-- @name		Список запросов в БД SQLite
--

--
-- Структура таблицы `categories_files`
--

CREATE TABLE categories_files ( 
	file_id INTEGER PRIMARY KEY,
	file_name VARCHAR(255),
	file_cat INTEGER NOT NULL DEFAULT '0',
	file_link TEXT,
	file_path TEXT,
	file_user INTEGER,
	file_added INT,
	file_priority TINYINT(1) NOT NULL DEFAULT '1',
	file_dl_id VARCHAR(32),
	file_dl_start INT(10),
	file_dl_last_start INT(10),
	file_dl_stop INT(10),
	file_dl_left INT(20),
	file_dl_time INT(10),
	file_dl_speed INTEGER,
	file_dl_range TINYINT(1) NOT NULL DEFAULT '0',
	file_dl_module VARCHAR(255),
	file_dl_bandwidth FLOAT(20),
	file_state VARCHAR(10) NOT NULL DEFAULT 'idle',
	file_error TINYINT(1) NOT NULL DEFAULT '0',
	file_desc TEXT,
	file_size INT(20),
	file_shared TINYINT(1) NOT NULL DEFAULT '0'
);
CREATE INDEX categories_files_file_name ON categories_files(file_name);
CREATE INDEX categories_files_file_cat ON categories_files(file_cat);
CREATE INDEX categories_files_file_user ON categories_files(file_user);
CREATE INDEX categories_files_file_size ON categories_files(file_size);
CREATE INDEX categories_files_file_priority ON categories_files(file_priority);
CREATE INDEX categories_files_file_dl_start ON categories_files(file_dl_start);
CREATE INDEX categories_files_file_dl_stop ON categories_files(file_dl_stop);
CREATE INDEX categories_files_file_dl_left ON categories_files(file_dl_left);
CREATE INDEX categories_files_file_dl_time ON categories_files(file_dl_time);
CREATE INDEX categories_files_file_added ON categories_files(file_added);

-- --------------------------------------------------------

--
-- Структура таблицы `categories_list`
--

CREATE TABLE categories_list (
	cat_id INTEGER PRIMARY KEY,
	cat_name VARCHAR(255),
	cat_path TEXT,
	cat_icon VARCHAR(10),
	cat_user INT(10),
	cat_root INT(10),
	cat_desc TEXT,
	cat_types TEXT
);

-- --------------------------------------------------------

--
-- Структура таблицы `domains_list`
--

CREATE TABLE domains_list (
	domain_id INTEGER PRIMARY KEY,
	domain_name VARCHAR(255),
	domain_user VARCHAR(255),
	domain_pass VARCHAR(255),
	domain_share TINYINT(1) NOT NULL DEFAULT '0'
);

-- --------------------------------------------------------

--
-- Структура таблицы `languages`
--

CREATE TABLE languages (
	lang_id INTEGER PRIMARY KEY,
	lang_key VARCHAR(2),
	lang_name VARCHAR(255),
	lang_default TINYINT(1) NOT NULL DEFAULT '0',
	lang_authors TEXT,
	lang_links TEXT
);
CREATE INDEX languages_lang_name ON languages(lang_name);

-- --------------------------------------------------------

--
-- Структура таблицы `modules_list`
--

CREATE TABLE modules_list (
	module_id INTEGER PRIMARY KEY,
	module_key VARCHAR(255),
	module_name VARCHAR(255),
	module_author VARCHAR(255),
	module_url VARCHAR(255),
	module_version VARCHAR(10),
	module_engine_author VARCHAR(255),
	module_engine_url VARCHAR(255),
	module_engine_version_support VARCHAR(255),
	module_engine_version_current VARCHAR(255),
	module_default TINYINT(1) NOT NULL DEFAULT '0',
	module_enabled TINYINT(1) NOT NULL DEFAULT '0'
);
CREATE INDEX modules_list_module_name ON modules_list(module_name);
CREATE UNIQUE INDEX modules_list_module_id ON modules_list(module_id);

-- --------------------------------------------------------

--
-- Структура таблицы `modules_settings`
--

CREATE TABLE modules_settings (
	setting_id INTEGER PRIMARY KEY,
	setting_module INT(5),
	setting_key VARCHAR(255),
	setting_default TEXT,
	setting_value TEXT,
	setting_type VARCHAR(10),
	setting_position INT(3) NOT NULL,
	setting_actions TEXT
);
CREATE INDEX modules_settings_setting_module ON modules_settings(setting_module);
CREATE INDEX modules_settings_setting_key ON modules_settings(setting_key);
CREATE INDEX modules_settings_setting_position ON modules_settings(setting_position);
CREATE UNIQUE INDEX modules_settings_setting_id ON modules_settings(setting_id);

-- --------------------------------------------------------

--
-- Структура таблицы `modules_versions`
--

CREATE TABLE modules_versions (
  version_id INTEGER PRIMARY KEY,
  version_module VARCHAR(10),
  version_min VARCHAR(10)
);
CREATE INDEX modules_versions_version_module ON modules_versions(version_module);
CREATE UNIQUE INDEX modules_versions_version_id ON modules_versions(version_id);

-- --------------------------------------------------------

--
-- Структура таблицы `schedule_events`
--

CREATE TABLE schedule_events (
	event_id INTEGER PRIMARY KEY,
	event_interlace VARCHAR(10),
	event_user INT(10),
	event_time INT(10),
	event_type TINYINT(1) NOT NULL DEFAULT '0',
	event_state VARCHAR(10) NOT NULL DEFAULT 'blocked',
	event_files TEXT
);
CREATE INDEX schedule_events_event_state ON schedule_events(event_state);
CREATE INDEX schedule_events_event_time ON schedule_events(event_time);
CREATE INDEX schedule_events_event_user ON schedule_events(event_user);

-- --------------------------------------------------------

--
-- Структура таблицы `schedule_time`
--

CREATE TABLE schedule_time (
	time_id INTEGER PRIMARY KEY,
	time_users TEXT,
	time_start VARCHAR(11),
	time_end VARCHAR(11),
	time_every TINYINT(1),
	time_interlace TINYINT(1) NOT NULL DEFAULT '0',
	time_allow TINYINT(1) NOT NULL DEFAULT '1'
);
CREATE INDEX schedule_time_time_start ON schedule_time(time_start);

-- --------------------------------------------------------

--
-- Структура таблицы `settings_groups`
--

CREATE TABLE settings_groups (
	group_id INTEGER PRIMARY KEY,
	group_key VARCHAR(255)
);
CREATE INDEX settings_groups_group_key ON settings_groups(group_key);

-- --------------------------------------------------------

--
-- Структура таблицы `settings_list`
--

CREATE TABLE settings_list (
	setting_id INTEGER PRIMARY KEY,
	setting_group INT(5) NOT NULL DEFAULT '0',
	setting_position INT(5) NOT NULL DEFAULT '0',
	setting_key VARCHAR(255),
	setting_default TEXT,
	setting_value TEXT,
	setting_type VARCHAR(10),
	setting_db VARCHAR(10),
	setting_options TEXT,
	setting_actions TEXT
);

-- --------------------------------------------------------

--
-- Структура таблицы `system_cache`
--

CREATE TABLE system_cache (
	cache_id INTEGER PRIMARY KEY,
	cache_name VARCHAR(255),
	cache_value TEXT,
	cache_uid VARCHAR(32),
	cache_added INT(10),
	cache_priority TINYINT(1) NOT NULL DEFAULT '1',
	cache_array TINYINT(1) NOT NULL DEFAULT '0'
);
CREATE INDEX system_cache_cache_name ON system_cache(cache_name);
CREATE INDEX system_cache_cache_uid ON system_cache(cache_uid);
CREATE INDEX system_cache_cache_added ON system_cache(cache_added);
CREATE INDEX system_cache_cache_priority ON system_cache(cache_priority);

-- --------------------------------------------------------

--
-- Структура таблицы `system_log`
--

CREATE TABLE system_log (
	log_id INTEGER PRIMARY KEY,
	log_type VARCHAR(5),
	log_system TINYINT(1) NOT NULL DEFAULT '0',
	log_user INT(10),
	log_time INT(10),
	log_code VARCHAR(10),
	log_misc TEXT,
	log_visible TINYINT(1) NOT NULL DEFAULT '1'
);
CREATE INDEX system_log_log_system ON system_log(log_system);
CREATE INDEX system_log_log_user ON system_log(log_user);
CREATE INDEX system_log_log_time ON system_log(log_time);
CREATE INDEX system_log_log_code ON system_log(log_code);
CREATE INDEX system_log_log_visible ON system_log(log_visible);
CREATE INDEX system_log_log_type ON system_log(log_type);

-- --------------------------------------------------------

--
-- Структура таблицы `users_list`
--

CREATE TABLE users_list (
	user_id INTEGER PRIMARY KEY,
	user_name VARCHAR(255),
	user_pass VARCHAR(32),
	user_admin TINYINT(1) NOT NULL DEFAULT '0',
	user_lang VARCHAR(2),
	user_max_speed INTEGER(5) NOT NULL DEFAULT '128',
	user_max_amount INTEGER NOT NULL DEFAULT '10'
);

-- --------------------------------------------------------