<?php

/**
* @package		ADOS - Automatic Downloading System
* @version		1.3.1 (build 63)
*
* @author		DINI
* @copyright	2007—2008
*
* @name			Модуль Axel
*/

/**
* Класс, содержащий базовый набор функций
* для работы с модулем Axel.
*
* @author	DINI
* @version	1.1.5
*/

class module_axel
{
	/**
	* Путь до исполняемого файла программы
	*
	* @var string
	*/
	
	var $path 		= "";
	
	/**
	* Текст ошибки
	*
	* @var string
	*/

	var $error 		= "";
	
	/**
	* Настройки модуля
	*
	* @var array
	*/

	var $settings 	= array();
	
	/**
	* Текущее состояние закачки
	*
	* @var string
	*/

	var $state		= array(	'file_dl_left'	=> 0,
								'file_dl_time'	=> 0,
								'file_dl_speed'	=> 0,
								);
	
	/**
	* Ссылка на переадресацию
	*
	* @var string
	*/

	var $redirect	= "";
	
	/**
	* Имя файла
	*
	* @var string
	*/

	var $filename	= "";
	
	/**
	* Конструктор класса (не страндартный PHP)
	* 
	* Загружает языковые строки для модуля.
	* Всегда возвращает TRUE.
	* 
	* @return	bool			Отметка об успешном выполнении
	*/
	
	function __class_construct()
	{
		$this->engine->load_lang( 'module_axel' );
		
		return TRUE;
	}
	
	/**
	* Получение версии программы (стандартная функция)
	* 
	* Делает запрос для получения версии
	* обслуживаемой модулем программы.
	* 
	* @return	string			Номер версии
	* @return 	bool			FALSE
	*/
	
	function std_get_program_version()
	{
		exec( escapeshellcmd( "{$this->path} -V" ), $output );
		
		if( !preg_match( "#axel version ([0-9a-zA-Z\.]+)#i", implode( "", $output ), $match ) )
		{
			$this->error =& $this->engine->lang['error_module_cant_get_version'];
			
			$this->engine->add_log_event( 1, "EMA_001" );
			
			return FALSE;
		}
		
		return $match[1];
	}
	
	/**
	* Запуск закачки
	* 
	* Генерирует необходимую строку для автоматического
	* старта закачки CRON'ом.
	* 
	* @param 	array			Параметры закачиваемого файла
	* 
	* @return	array			( 'size'	=> Размер файла,
	* 							  'string'	=> Строка для запуска
	* 							  )
	* @return 	bool			FALSE
	*/
	
	function std_download_start( $file )
	{
		//-------------------------------------------------
		// Изменяем адрес ссылки для применения авторизации
		//-------------------------------------------------
		
		$link = $this->redirect ? $this->redirect : $file['file_link'];
		$link = escapeshellcmd( preg_replace( "#\s+#", "%20", urldecode( $link ) ) );
		$link = str_replace( array( "'", '"' ), array( "\'", '\"' ), $link );
		$link = str_replace( array( "\\'", '\\"' ), array( "\'", '\"' ), $link );
		
		if( $this->auth ) $link = preg_replace( "#^(http|ftp)://#", "\\1://".escapeshellcmd( $this->auth ), $link );
		
		//-------------------------------------------------
		// Определяем количество потоков в зависимости от
		// настроек модуля и размера файла
		//-------------------------------------------------
		
		if( $file['file_size'] == 0 )
		{
			$this->error = &$this->engine->lang['error_null_file_size'];
			
			$this->engine->add_log_event( 1, "EMA_002" );
			
			return FALSE;
		}
		else if( $file['file_size'] <= 512000 or $this->settings['threads_num'] == 1 )
		{
			$threads_num = 1; 
		}
		else if( $file['file_size'] <= 1048576 or $this->settings['threads_num'] == 2 )
		{
			$threads_num = 2; 
		}
		else if( $this->settings['threads_num'] == 3 )
		{
			$threads_num = 3;
		}
		else if( $file['file_size'] <= 2097152 or $this->settings['threads_num'] == 4 )
		{
			$threads_num = 4; 
		}
		else if( $this->settings['threads_num'] == 5 )
		{
			$threads_num = 5;
		}
		else if( $file['file_size'] <= 4194304 or $this->settings['threads_num'] == 6 )
		{
			$threads_num = 6; 
		}
		else if( $this->settings['threads_num'] == 7 )
		{
			$threads_num = 7;
		}
		else if( $file['file_size'] <= 10485760 or $this->settings['threads_num'] == 8 )
		{
			$threads_num = 8; 
		}
		else if( $this->settings['threads_num'] == 9 )
		{
			$threads_num = 9;
		}
		else 
		{
			$threads_num = 10;
		}
		
		$this->engine->add_log_event( 4, "IMA_001", array( 'file_size' => $file['file_size'], 'threads_num' => $threads_num ) );
		
		//-------------------------------------------------
		// Формируем и возвращаем строку для начала закачки
		//-------------------------------------------------
		
		return array( 'size'	=> $file['file_size'],
					  'string'	=> $this->settings['engine_path']." -n {$threads_num} -o {$this->engine->config['save_path']}/_tmp/{$file['file_id']}_{$file['file_user']}_{$file['file_dl_id']}.ados -a {$link}",
					  'name'	=> $this->filename,
					  'speed'	=> "--max-speed",
					  );
	}
	
	/**
	* Возобновление закачки
	* 
	* Алиас функции запуска закачки.
	* Для данной функции необходимо наличие идентификатора файла.
	* 
	* @param 	array			Параметры закачиваемого файла
	* @param 	mixed			Параметры авторизации
	* 
	* @return	string			Строка для запуска
	* @return 	bool			FALSE
	*/
	
	function std_download_continue( $file )
	{
		return $this->std_download_start( &$file );
	}
	
	/**
	* Приостановка закачки
	* 
	* Алиас функции остановки закачки.
	* 
	* @param 	array			Параметры закачиваемого файла
	* 
	* @return 	bool			Результат выполнения операции
	*/
	
	function std_download_pause( $file )
	{
		return $this->std_download_stop( &$file, FALSE );
	}
	
	/**
	* Остановка закачки
	* 
	* Выполняет поиск процессов, осуществляющих закачку, и
	* отменяет их. Также удаляет CRON файл, если он
	* существует.
	* 
	* @param 	array			Параметры закачиваемого файла
	* @param 	bool			Удалить лог закачки
	* 
	* @return 	bool			Результат выполнения операции
	*/
	
	function std_download_stop( $file, $delete_log=TRUE )
	{
		if( !preg_match( "#^[a-zA-Z0-9]{32}$#", $file['file_dl_id'] ) )
		{
			$this->error =& $this->engine->lang['error_wrong_file_dl_id'];
			
			return FALSE;
		}
		
		//-------------------------------------------------
		// Останавливаем закачку
		//-------------------------------------------------
		
		$output = shell_exec( "ps -ww" );
		
		if( preg_match_all( "#\s*(\d+).+{$file['file_dl_id']}\.ados#", $output, $match, PREG_SET_ORDER ) ) foreach( $match as $pid )
		{
			$pids[] = $pid[1];
		}
		
		if( is_array( $pids ) ) exec( "kill ".implode( " ", $pids ) );
		
		//-------------------------------------------------
		// Удаляем CRON файл
		//-------------------------------------------------
		
		if( $this->engine->classes['downloader']->_delete_cron_file( $file['file_dl_id'] ) === FALSE )
		{
			return FALSE;
		}
		
		//-------------------------------------------------
		// Удаляем лог закачки
		//-------------------------------------------------
		
		if( $delete_log === TRUE )
		{
			@unlink( $this->engine->config['save_path']."/_log/{$file['file_dl_id']}.log" );
		}
		
		return TRUE;
	}
	
	/**
	* Проверка состояния закачки
	* 
	* Выполняет поиск процессов, осуществляющих закачку.
	* Если они не найдены, то проверяет, закачан ли файл
	* полностью.
	* 
	* @param 	array			Параметры закачиваемого файла
	* 
	* @return 	bool			Результат выполнения операции
	*/
	
	function std_download_is_running( $file )
	{
		if( !preg_match( "#^[a-zA-Z0-9]{32}$#", $file['file_dl_id'] ) )
		{
			$this->error = 2;
			
			$this->engine->add_log_event( 1, "EMA_003", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
			
			return FALSE;
		}
		
		//-------------------------------------------------
		// Проверяем процессы
		//-------------------------------------------------
		
		$output = shell_exec( "ps -ww" );
		
		if( preg_match_all( "#\s*(\d+).+{$file['file_dl_id']}\.ados#", $output, $match, PREG_SET_ORDER ) ) foreach( $match as $pid )
		{
			$pids[] = $pid[1];
		}
		
		if( is_array( $pids ) )
		{
			$this->engine->add_log_event( 4, "IMA_002", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
			
			//-------------------------------------------------
			// Проверяем, нет ли простоя
			//-------------------------------------------------
			
			if( $this->engine->config['restart_after'] )
			{
				return $this->engine->classes['downloader']->download_restart( &$file );
			}
		}
		
		//-------------------------------------------------
		// Процессов нет, проверяем скачан ли файл
		//-------------------------------------------------
		
		$this->std_get_download_info( &$file );
		
		if( $this->state['file_dl_left'] )
		{
			$this->error = 1;
			
			$this->engine->add_log_event( 3, "IMA_003", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
		}
		
		$this->engine->add_log_event( 4, "IMA_004", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
		
		return FALSE;
	}
	
	/**
	* Обновление параметров закачки
	* 
	* Считывает и записывает информацию из журнала
	* закачки.
	* 
	* @param 	string			Параметры закачиваемого файла
	* 
	* @return 	bool			Результат выполнения операции
	*/
	
	function std_get_download_info( $file )
	{
		$this->_clear_state();
		
		$save = $this->engine->config['save_path']."/_tmp/{$file['file_id']}_{$file['file_user']}_{$file['file_dl_id']}.ados";
		
		//-------------------------------------------------
		// Проверяем переданный идентификатор
		//-------------------------------------------------
		
		if( !preg_match( "#^[a-zA-Z0-9]{32}$#", $file['file_dl_id'] ) )
		{
			$this->error =& $this->engine->lang['error_wrong_file_dl_id'];
			
			$this->engine->add_log_event( 1, "EMA_004", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
			
			return FALSE;
		}
		
		//-------------------------------------------------
		// Проверяем наличие журнала закачки
		//-------------------------------------------------
		
		$log = $this->engine->config['save_path']."/_log/{$file['file_dl_id']}.log";
		
		if( !file_exists( $log ) )
		{
			$exists = file_exists( $save );
			
			$this->state['file_dl_left']  = $exists ? $file['file_size'] - filesize( $save ) : $file['file_size'];
			$this->state['file_dl_time']  = $exists ? 0 : $file['file_dl_time'];
			$this->state['file_dl_speed'] = $exists ? 0 : $file['file_dl_speed'];
			
			$this->engine->add_log_event( 4, "IMA_005", array(	'file_id'		=> $file['file_id'],
																'file_name'		=> $file['file_name'],
																'file_dl_id'	=> $file['file_dl_id'],
																'file_dl_left'	=> $this->state['file_dl_left'],
																'file_dl_time'	=> $this->state['file_dl_time'],
																'file_dl_speed'	=> $this->state['file_dl_speed']
																)	);
			
			return TRUE;
		}
		
		//-------------------------------------------------
		// Считываем первые строки с информацией
		//-------------------------------------------------
		
		$string = file_get_contents( $log, NULL, NULL, 0, 1024 );
		
		if( strpos( $string, "Server unsupported" ) !== FALSE )
		{
			$this->std_download_stop( $file );
			
			$this->error =& $this->engine->lang['error_server_unsupported'];
			
			$this->engine->add_log_event( 1, "EMA_005", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
			
			return FALSE;
		}
		
		//-------------------------------------------------
		// Считываем последнюю строку с информацией
		//-------------------------------------------------
		
		$string = file_get_contents( $log, NULL, NULL, filesize( $log ) - 1024, 1024 );
		
		preg_match_all( "#(^|\r)\[\s{0,2}(\d{1,3})%\].+\[\s*(\d+\.\d{1})(B|KB|MB)/s\] \[\s*(\d{1,2})(d|h|:)(\d{1,2})\]($|\r)#i", $string, $match, PREG_SET_ORDER );
		
		$match = $match[ count( $match ) - 1 ];
		
		//-------------------------------------------------
		// Обрабатываем полученные параметры
		//-------------------------------------------------
		
		if( !is_array( $match ) )
		{
			$this->error =& $this->engine->lang['error_cant_update_info'];
			
			$this->engine->add_log_event( 1, "EMA_006", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
			
			return FALSE;
		}
		
		$this->state['file_dl_left'] = floor( $file['file_size'] - $file['file_size'] * $match[2] / 100 );
		
		if( file_exists( $save ) and filesize( $save ) == $file['file_size'] ) $this->state['file_dl_left'] = 0;
		
		switch( strtolower( $match[6] ) )
		{
			case ':':
				$this->state['file_dl_time'] = intval( $match[5] ) * 60 + intval( $match[7] );
				break;
				
			case 'h':
				$this->state['file_dl_time'] = intval( $match[5] ) * 60 * 60 + intval( $match[7] ) * 60;
				break;
				
			case 'd':
				$this->state['file_dl_time'] = intval( $match[5] ) * 60 * 60 * 24 + intval( $match[7] ) * 60 * 60;
				break;
		}
		
		switch( strtolower( $match[4] ) )
		{
			case 'b':
				$this->state['file_dl_speed'] = floor( floatval( $match[3] ) );
				break;
				
			case 'kb':
				$this->state['file_dl_speed'] = floor( floatval( $match[3] ) * 1024 );
				break;
				
			case 'mb':
				$this->state['file_dl_speed'] = floor( floatval( $match[3] ) * 1024 * 1024 );
				break;
		}
		
		//-------------------------------------------------
		// Сохраняем информацию
		//-------------------------------------------------
		
		$this->engine->add_log_event( 4, "IMA_006", array(	'file_id'		=> $file['file_id'],
															'file_name'		=> $file['file_name'],
															'file_dl_id'	=> $file['file_dl_id'],
															'file_dl_left'	=> $this->state['file_dl_left'],
															'file_dl_time'	=> $this->state['file_dl_time'],
															'file_dl_speed'	=> $this->state['file_dl_speed']
															)	);
	}
	
	//-------------------------------------------------
	
	/**
	* Очистка информации о файле
	* 
	* Очищает массив с информацией о текущем файле с тем,
	* чтобы в нем не осталась информация о предыдущем файле.
	* 
	* @return	void
	*/
	
	function _clear_state()
	{
		$this->state = array(	'file_dl_left'	=> 0,
								'file_dl_time'	=> 0,
								'file_dl_speed'	=> 0,
								);
	}
}

?>