<?php

/**
* @package		ADOS - Automatic Downloading System
* @version		1.3.1 (build 63)
*
* @author		DINI
* @copyright	2007—2008
*
* @name			Модуль cURL
*/

/**
* Класс, содержащий базовый набор функций
* для работы с модулем cURL.
*
* @author	DINI
* @version	1.1.5
*/

class module_curl
{
	/**
	* Путь до исполняемого файла программы
	*
	* @var string
	*/
	
	var $path 		= "";
	
	/**
	* Текст ошибки
	*
	* @var string
	*/

	var $error 		= "";
	
	/**
	* Настройки модуля
	*
	* @var array
	*/

	var $settings 	= array();
	
	/**
	* Текущее состояние закачки
	*
	* @var string
	*/

	var $state		= array(	'file_dl_left'	=> 0,
								'file_dl_time'	=> 0,
								'file_dl_speed'	=> 0,
								);
	
	/**
	* Ссылка на переадресацию
	*
	* @var string
	*/

	var $redirect	= "";
	
	/**
	* Имя файла
	*
	* @var string
	*/

	var $filename	= "";
	
	/**
	* Конструктор класса (не страндартный PHP)
	* 
	* Загружает языковые строки для модуля.
	* Всегда возвращает TRUE.
	* 
	* @return	bool			Отметка об успешном выполнении
	*/
	
	function __class_construct()
	{
		$this->engine->load_lang( 'module_curl' );
		
		return TRUE;
	}
	
	/**
	* Получение версии программы (стандартная функция)
	* 
	* Делает запрос для получения версии
	* обслуживаемой модулем программы.
	* 
	* @return	string			Номер версии
	* @return 	bool			FALSE
	*/
	
	function std_get_program_version()
	{
		exec( escapeshellcmd( "{$this->path} -V" ), $output );
		
		if( !preg_match( "#curl ([0-9a-zA-Z\.]+)#i", implode( "", $output ), $match ) )
		{
			$this->error =& $this->engine->lang['error_module_cant_get_version'];
			
			$this->engine->add_log_event( 1, "EMC_001" );
			
			return FALSE;
		}
		
		return $match[1];
	}
	
	/**
	* Запуск закачки
	* 
	* Генерирует необходимую строку для автоматического
	* старта закачки CRON'ом.
	* 
	* @param 	array			Параметры закачиваемого файла
	* 
	* @return	array			( 'size'	=> Размер файла,
	* 							  'string'	=> Строка для запуска
	* 							  )
	* @return 	bool			FALSE
	*/
	
	function std_download_start( $file )
	{
		//-------------------------------------------------
		// Изменяем адрес ссылки для применения авторизации
		//-------------------------------------------------
		
		$link = $this->redirect ? $this->redirect : $file['file_link'];
		$link = escapeshellcmd( preg_replace( "#\s+#", "%20", urldecode( $link ) ) );
		$link = str_replace( array( "'", '"' ), array( "\'", '\"' ), $link );
		$link = str_replace( array( "\\'", '\\"' ), array( "\'", '\"' ), $link );
		
		if( $this->auth ) $link .= " -u ".escapeshellcmd( preg_replace( "#@$#", "", $this->auth ) );
		
		//-------------------------------------------------
		// Проверяем, не является ли размер файла нулевым
		//-------------------------------------------------
		
		if( $file['file_size'] == 0 )
		{
			$this->error = &$this->engine->lang['error_null_file_size'];
			
			$this->engine->add_log_event( 1, "EMC_002", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
			
			return FALSE;
		}
		
		$this->engine->add_log_event( 4, "IMC_001", array( 'file_size' => $file['file_size'] ) );
		
		//-------------------------------------------------
		// Формируем и возвращаем строку для начала закачки
		//-------------------------------------------------
		
		return array( 'size'	=> $file['file_size'],
					  'string'	=> $this->settings['engine_path']." {$link} -o {$this->engine->config['save_path']}/_tmp/{$file['file_id']}_{$file['file_user']}_{$file['file_dl_id']}.ados",
					  'name'	=> $this->filename,
					  'speed'	=> "--limit-rate",
					  );
	}
	
	/**
	* Возобновление закачки
	* 
	* Вызывает функцию начала закачки и изменяет полученную
	* строку с параметрами запуска для автоматического
	* возобновления закачки модулем.
	* Для данной функции необходимо наличие идентификатора файла.
	* 
	* @param 	array			Параметры закачиваемого файла
	* @param 	mixed			Параметры авторизации
	* 
	* @return	string			Строка для запуска
	* @return 	bool			FALSE
	*/
	
	function std_download_continue( $file )
	{
		$params = $this->std_download_start( &$file );
		
		$params['string'] .= " -C -";
		
		return $params;
	}
	
	/**
	* Приостановка закачки
	* 
	* Алиас функции остановки закачки.
	* 
	* @param 	array			Параметры закачиваемого файла
	* 
	* @return 	bool			Результат выполнения операции
	*/
	
	function std_download_pause( $file )
	{
		return $this->std_download_stop( &$file, FALSE );
	}
	
	/**
	* Остановка закачки
	* 
	* Выполняет поиск процессов, осуществляющих закачку, и
	* отменяет их. Также удаляет CRON файл, если он
	* существует.
	* 
	* @param 	array			Параметры закачиваемого файла
	* @param 	bool			Удалить лог закачки
	* 
	* @return 	bool			Результат выполнения операции
	*/
	
	function std_download_stop( $file, $delete_log=TRUE )
	{
		if( !preg_match( "#^[a-zA-Z0-9]{32}$#", $file['file_dl_id'] ) )
		{
			$this->error =& $this->engine->lang['error_wrong_file_dl_id'];
			
			return FALSE;
		}
		
		//-------------------------------------------------
		// Останавливаем закачку
		//-------------------------------------------------
		
		$output = shell_exec( "ps -ww" );
		
		if( preg_match_all( "#\s*(\d+).+{$file['file_dl_id']}\.ados#", $output, $match, PREG_SET_ORDER ) ) foreach( $match as $pid )
		{
			$pids[] = $pid[1];
		}
		
		if( is_array( $pids ) ) exec( "kill ".implode( " ", $pids ) );
		
		//-------------------------------------------------
		// Удаляем CRON файл
		//-------------------------------------------------
		
		if( $this->engine->classes['downloader']->_delete_cron_file( $file['file_dl_id'] ) === FALSE )
		{
			return FALSE;
		}
		
		//-------------------------------------------------
		// Удаляем лог закачки
		//-------------------------------------------------
		
		if( $delete_log === TRUE )
		{
			@unlink( $this->engine->config['save_path']."/_log/{$file['file_dl_id']}.log" );
		}
		
		return TRUE;
	}
	
	/**
	* Проверка состояния закачки
	* 
	* Выполняет поиск процессов, осуществляющих закачку.
	* Если они не найдены, то проверяет, закачан ли файл
	* полностью.
	* 
	* @param 	array			Параметры закачиваемого файла
	* 
	* @return 	bool			Результат выполнения операции
	*/
	
	function std_download_is_running( $file )
	{
		if( !preg_match( "#^[a-zA-Z0-9]{32}$#", $file['file_dl_id'] ) )
		{
			$this->error = 2;
			
			$this->engine->add_log_event( 1, "EMC_003", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
			
			return FALSE;
		}
		
		//-------------------------------------------------
		// Проверяем процессы
		//-------------------------------------------------
		
		$output = shell_exec( "ps -ww" );
		
		if( preg_match_all( "#\s*(\d+).+{$file['file_dl_id']}\.ados#", $output, $match, PREG_SET_ORDER ) ) foreach( $match as $pid )
		{
			$pids[] = $pid[1];
		}
		
		if( is_array( $pids ) )
		{
			$this->engine->add_log_event( 4, "IMC_002", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
			
			//-------------------------------------------------
			// Проверяем, нет ли простоя
			//-------------------------------------------------
			
			if( $this->engine->config['restart_after'] )
			{
				return $this->engine->classes['downloader']->download_restart( &$file );
			}
		}
		
		//-------------------------------------------------
		// Процессов нет, проверяем скачан ли файл
		//-------------------------------------------------
		
		$this->std_get_download_info( &$file );
		
		if( $this->state['file_dl_left'] )
		{
			$this->error = 1;
			
			$this->engine->add_log_event( 3, "IMC_003", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
		}
		
		$this->engine->add_log_event( 4, "IMC_004", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
		
		return FALSE;
	}
	
	/**
	* Обновление параметров закачки
	* 
	* Считывает и записывает информацию из журнала
	* закачки.
	* 
	* @param 	string			Параметры закачиваемого файла
	* 
	* @return 	bool			Результат выполнения операции
	*/
	
	function std_get_download_info( $file )
	{
		$this->_clear_state();
		
		$save = $this->engine->config['save_path']."/_tmp/{$file['file_id']}_{$file['file_user']}_{$file['file_dl_id']}.ados";
		
		//-------------------------------------------------
		// Проверяем переданный идентификатор
		//-------------------------------------------------
		
		if( !preg_match( "#^[a-zA-Z0-9]{32}$#", $file['file_dl_id'] ) )
		{
			$this->error =& $this->engine->lang['error_wrong_file_dl_id'];
			
			$this->engine->add_log_event( 1, "EMC_004", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
			
			return FALSE;
		}
		
		//-------------------------------------------------
		// Проверяем наличие журнала закачки
		//-------------------------------------------------
		
		$log = $this->engine->config['save_path']."/_log/{$file['file_dl_id']}.log";
		
		if( !file_exists( $log ) )
		{
			$exists = file_exists( $save );
			
			$this->state['file_dl_left']  = $exists ? $file['file_size'] - filesize( $save ) : $file['file_size'];
			$this->state['file_dl_time']  = $exists ? 0 : $file['file_dl_time'];
			$this->state['file_dl_speed'] = $exists ? 0 : $file['file_dl_speed'];
			
			$this->engine->add_log_event( 4, "IMC_005", array(	'file_id'		=> $file['file_id'],
																'file_name'		=> $file['file_name'],
																'file_dl_id'	=> $file['file_dl_id'],
																'file_dl_left'	=> $this->state['file_dl_left'],
																'file_dl_time'	=> $this->state['file_dl_time'],
																'file_dl_speed'	=> $this->state['file_dl_speed']
																)	);
			
			return TRUE;
		}
		
		//-------------------------------------------------
		// Считываем первые строки с информацией
		//-------------------------------------------------
		
		$string = file_get_contents( $log, NULL, NULL, 0, 1024 );
		
		preg_match( "#Resuming transfer from byte position (\d+)#", $string, $pos );
		
		if( preg_match( "#^(\(curl\) \(\d+\) .*)$#m", $string, $error ) )
		{
			$this->std_download_stop( $file );
			
			$this->error =& $this->engine->lang['error_internal'];
			
			$this->engine->add_log_event( 1, "EMC_005", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'], 'error_desc' => $match[1] ) );
			
			return FALSE;
		}
		
		//-------------------------------------------------
		// Считываем последнюю строку с информацией
		//-------------------------------------------------
		
		$string = file_get_contents( $log, NULL, NULL, filesize( $log ) - 1024, 1024 );
		
		preg_match_all( "#([0-9\.]+)([kmg]?)\s+\d+\s+\d+\s+[0-9\.]+[kmg]?\s+\d+\s+([\d-]{1,2}:[\d-]{2}:[\d-]{2}|\d+d\s+\d+h)\s+([\d-]{1,2}:[\d-]{2}:[\d-]{2}|\d+d\s+\d+h)\s+(([\d-]{1,2}):([\d-]{2}):([\d-]{2})|(\d+)(d)\s+(\d+)h)\s+(\d+)([kmg]?)#i", $string, $match, PREG_SET_ORDER );
		
		$match = $match[ count( $match ) - 1 ];
		
		//-------------------------------------------------
		// Обрабатываем полученные параметры
		//-------------------------------------------------
		
		if( !is_array( $match ) )
		{
			$this->error =& $this->engine->lang['error_cant_update_info'];
			
			$this->engine->add_log_event( 1, "EMC_006", array( 'file_id' => $file['file_id'], 'file_name' => $file['file_name'], 'file_dl_id' => $file['file_dl_id'] ) );
			
			return FALSE;
		}
		
		switch( strtolower( $match[2] ) )
		{
			case 'k': $match[1] *= 1024;
			break;
			
			case 'm': $match[1] *= 1048576;
			break;
			
			case 'g': $match[1] *= 1073741824;
			break;
		}
		
		if( $pos[1] ) $match[1] += $pos[1];
		
		$this->state['file_dl_left'] = floor( $file['file_size'] - $match[1] );
		
		if( file_exists( $save ) and filesize( $save ) == $file['file_size'] ) $this->state['file_dl_left'] = 0;
		
		$match[6] = $match[6] == "--" ? 0 : $match[10] == "d" ? $match[9] * 24 : $match[6];
		$match[7] = $match[7] == "--" ? 0 : $match[10] == "d" ? $match[11] : $match[7];
		$match[8] = $match[8] == "--" ? 0 : $match[10] == "d" ? 0 : $match[8];
		
		$this->state['file_dl_time'] = $match[6] * 60 * 60 + $match[7] * 60 + $match[8];
		
		switch( strtolower( $match[13] ) )
		{
			case 'k': 
				$this->state['file_dl_speed'] = floor( $match[12] * 1024 );
				break;
			
			case 'm':
				$this->state['file_dl_speed'] = floor( $match[12] * 1048576 );
				break;
			
			case 'g':
				$this->state['file_dl_speed'] = floor( $match[12] * 1073741824 );
				break;

			default:
				$this->state['file_dl_speed'] = $match[12];
				break;
		}
		
		//-------------------------------------------------
		// Сохраняем информацию
		//-------------------------------------------------
		
		$this->engine->add_log_event( 4, "IMC_006", array(	'file_id'		=> $file['file_id'],
															'file_name'		=> $file['file_name'],
															'file_dl_id'	=> $file['file_dl_id'],
															'file_dl_left'	=> $this->state['file_dl_left'],
															'file_dl_time'	=> $this->state['file_dl_time'],
															'file_dl_speed'	=> $this->state['file_dl_speed']
															)	);
	}
	
	//-------------------------------------------------
	
	/**
	* Очистка информации о файле
	* 
	* Очищает массив с информацией о текущем файле с тем,
	* чтобы в нем не осталась информация о предыдущем файле.
	* 
	* @return	void
	*/
	
	function _clear_state()
	{
		$this->state = array(	'file_dl_left'	=> 0,
								'file_dl_time'	=> 0,
								'file_dl_speed'	=> 0,
								);
	}
}

?>